% Program by Natalie on 04.08.11. It takes the raw data from acquiaition containing
% the time delays in the first row and the wavelengths in the first column
% not baseline or chirp corrected, for one data set. It does the baseline
% correction
% and the chirp cPEAorrection and puts the zero at the chosen point. It extracts
% the wanted timedelays (same as measured) and generates chirp-corrected timdelay curves and
% dynamics. This one puts the name in ps for the spectra (rounded to the nearest 10 fs).

%load data
clear;
filename=input('Please specify filename ','s');%give name of file
Data_all=[];
Data_all=importdata(filename);
nwavelengths=size(Data_all,1)-1;
ntimes=size(Data_all,2)-1;
Data_t(1:nwavelengths,1:ntimes)=Data_all(2:nwavelengths+1,2:ntimes+1);
Data_raw=Data_t';

%prepare timedelay
timedelay(1,:)=Data_all(1,2:ntimes+1);
Timedelay_ps=timedelay';
timedelay_fs(1,:)=timedelay(1,:)*100;
timedelay_fs =round(timedelay_fs);
timedelay_fs=timedelay_fs*10; % round to the nearest 10 fs
timedelay_fs=timedelay_fs';
save Timedelay_ps Timedelay_ps -ascii;

%prepare wavelength
Wavelength_nm(:,1)=Data_all(2:nwavelengths+1,1);
Wavelength_nm=Wavelength_nm.*10;
Wavelength_nm=round(Wavelength_nm);% round to the nearest 0.1 nm
Wavelength_nm=Wavelength_nm./10;
save Wavelength_nm Wavelength_nm -ascii;

%subtracts the average of the first "n_bg" negative spectra as the background
background=[];
Data=[];
n_bg=input('How many background spectra do you want to subtract? ');
background=mean(Data_raw(1:n_bg,1:nwavelengths));
for i=1:ntimes for j=1:nwavelengths
%Data(i,j)=Data_raw(i,j)-background(1,j);
Data(i,j)=Data_raw(i,j);% to uncomment only when one wants to correct the chirp on NON background subtracted data, without forgetting to comment line 41.
end
end

%give chirp parameters (exponential fit)
disp('Chirp parameters from exponential fit');
y0=input('indicate y0 ');
A=input('indicate A ');
tau=input('indicate tau ');
x0=input('indicate x0 '); % this is a constant automatically generated by Igor

%create shifted timedelays
td_x=[];
td_x(1:nwavelengths,1:ntimes)=0;
for i=1:nwavelengths
    zero=y0+A.*exp(-(Wavelength_nm(i)-x0)./tau);
    for j=1:ntimes
    td_x(i,j) = timedelay(j)-zero;
    end
end
td_x=td_x';

%interpolate for chirp correction
Data_chirp=[];
Data_chirp(1:ntimes,1:nwavelengths)=0;
for i=1:nwavelengths
Data_chirp(:,i)=interp1(td_x(:,i),Data(:,i),timedelay,'pchip');
end

%make output files
Prefix=input('Specify sample name ', 's');
timedelay_ps_1=timedelay_fs/1000;

% 1) raw spectra
Data_raw=Data_raw';
Spectra_label=[];
for i=1:ntimes
    Spectra_label=strvcat(Spectra_label, ['raw_',Prefix,'_',num2str(timedelay_ps_1(i)),'ps']);
end
Spectra_label=cellstr(Spectra_label);
Raw_name=[];
Raw_name=strvcat(Raw_name, ['Raw_',Prefix]);
outputfile = Raw_name;
fid = fopen(outputfile, 'w'); %open file for writing
for i = 1:ntimes
    LabelString = cell2mat(Spectra_label(i)); 
    fprintf(fid, '%s\t', LabelString);
end
fprintf(fid, '\n');
for j=1:nwavelengths 
    for i=1:ntimes
    fprintf(fid, '%e\t', Data_raw(j,i));
    end
    fprintf(fid, '\n');
end
fclose(fid);

% 2) baseline-corrected spectra
Data=Data';
Spectra_label=[];
for i=1:ntimes
    Spectra_label=strvcat(Spectra_label, ['bsl_',Prefix,'_',num2str(timedelay_ps_1(i)),'ps']);
end
Spectra_label=cellstr(Spectra_label);
Bsl_name=[];
Bsl_name=strvcat(Bsl_name, ['Bsl_',Prefix]);
outputfile = Bsl_name;
fid = fopen(outputfile, 'w'); %open file for writing
for i = 1:ntimes
    LabelString = cell2mat(Spectra_label(i)); 
    fprintf(fid, '%s\t', LabelString);
end
fprintf(fid, '\n');
for j=1:nwavelengths 
    for i=1:ntimes
    fprintf(fid, '%e\t', Data(j,i));
    end
    fprintf(fid, '\n');
end
fclose(fid);

% 3) chirp-corrected spectra
Data_chirp=Data_chirp';
Spectra_label=[];
for i=1:ntimes
    Spectra_label=strvcat(Spectra_label, ['c_',Prefix,'_',num2str(timedelay_ps_1(i)),'ps']);
end
Spectra_label=cellstr(Spectra_label);
Chirp_name=[];
Chirp_name=strvcat(Chirp_name, ['Chirp_',Prefix]);
outputfile = Chirp_name;
fid = fopen(outputfile, 'w'); %open file for writing
for i = 1:ntimes
    LabelString = cell2mat(Spectra_label(i)); 
    fprintf(fid, '%s\t', LabelString);
end
fprintf(fid, '\n');
for j=1:nwavelengths 
    for i=1:ntimes
    fprintf(fid, '%e\t', Data_chirp(j,i));
    end
    fprintf(fid, '\n');
end
fclose(fid);

% 4) baseline-corrected dynamics
Data=Data';
Dyn_label=[];
for i=1:nwavelengths
    Dyn_label=strvcat(Dyn_label, ['dyn_',Prefix,'_',num2str(Wavelength_nm(i)),'nm']);
end
Dyn_label=cellstr(Dyn_label);   
Dyn_name=[];
Dyn_name=strvcat(Dyn_name, ['Dyn_',Prefix]);
outputfile = Dyn_name;
fid = fopen(outputfile, 'w'); %open file for writing
for i = 1:nwavelengths
    LabelString = cell2mat(Dyn_label(i)); 
    fprintf(fid, '%s\t', LabelString);
end
fprintf(fid, '\n');
for j=1:ntimes 
    for i=1:nwavelengths
    fprintf(fid, '%e\t', Data(j,i));
    end
    fprintf(fid, '\n');
end
fclose(fid);

% 5) chirp-corrected dynamics
Data_chirp=Data_chirp';
Cdyn_label=[];
for i=1:nwavelengths
    Cdyn_label=strvcat(Cdyn_label, ['cdyn_',Prefix,'_',num2str(Wavelength_nm(i)),'nm']);
end
Cdyn_label=cellstr(Cdyn_label);   
Cdyn_name=[];
Cdyn_name=strvcat(Cdyn_name, ['Cdyn_',Prefix]);
outputfile = Cdyn_name;
fid = fopen(outputfile, 'w'); %open file for writing
for i = 1:nwavelengths
    LabelString = cell2mat(Cdyn_label(i)); 
    fprintf(fid, '%s\t', LabelString);
end
fprintf(fid, '\n');
for j=1:ntimes 
    for i=1:nwavelengths
    fprintf(fid, '%e\t', Data_chirp(j,i));
    end
    fprintf(fid, '\n');
end
fclose(fid);

% 5) chirp-corrected dynamics
TD_label=[];
for i=1:nwavelengths
    TD_label=strvcat(TD_label, ['td_',Prefix,'_',num2str(Wavelength_nm(i)),'nm']);
end
TD_label=cellstr(TD_label);   
TD_name=[];
TD_name=strvcat(TD_name, ['TD_',Prefix]);
outputfile = TD_name;
fid = fopen(outputfile, 'w'); %open file for writing
for i = 1:nwavelengths
    LabelString = cell2mat(TD_label(i)); 
    fprintf(fid, '%s\t', LabelString);
end
fprintf(fid, '\n');
for j=1:ntimes 
    for i=1:nwavelengths
    fprintf(fid, '%e\t', td_x(j,i));
    end
    fprintf(fid, '\n');
end
fclose(fid);
 
disp('For loading this data in IGOR use the following command:');
disp('loadwave/g/a/w');




